<div class="input-group input-group-narrow">
        {% include "django/forms/widgets/input.html" %}
    <div class="input-group-append">
        <span class="input-group-text">{{ currency_symbol }}</span>
    </div>
</div>
